import {App, Notice, PluginSettingTab, Setting} from 'obsidian';
import templateInstructions from './templates/templateInstructions.html';
import datetimeInstructions from './templates/datetimeInstructions.html';
import type { RaindropAPI } from './api';
import type RaindropPlugin from './main';
import CollectionsModal from './modal/collections';
import Renderer from './renderer';
import ApiTokenModal from './modal/apiTokenModal';

export class RaindropSettingTab extends PluginSettingTab {
	private plugin: RaindropPlugin;
	private api: RaindropAPI;
	private renderer: Renderer;

	constructor(app: App, plugin: RaindropPlugin, api: RaindropAPI) {
		super(app, plugin);
		this.plugin = plugin;
		this.renderer = new Renderer(plugin);
		this.api = api;
	}

	display(): void {
		const {containerEl} = this;

		containerEl.empty();
		if (this.plugin.settings.isConnected) {
			this.disconnect();
		} else {
			this.connect();
		}
		this.highlightsFolder();
		this.collections();
		this.autoSyncInterval();
		this.dateFormat();
		this.template();
		this.resetSyncHistory();
	}

	private connect(): void {
		new Setting(this.containerEl)
			.setName('Connect to Raindrop.io')
			.addButton((button) => {
				return button
					.setButtonText('Connect')
					.setCta()
					.onClick(async () => {
						const tokenModal = new ApiTokenModal(this.app, this.api);
						await tokenModal.waitForClose;

						if (this.api.tokenManager.get()) {
							new Notice('Token saved');
							const user = await this.api.getUser();
							this.plugin.settings.isConnected = true;
							this.plugin.settings.username = user.fullName;
							await this.plugin.saveSettings();
						}

						this.display(); // rerender
					});
			});
	}

	private async disconnect(): Promise<void> {
		new Setting(this.containerEl)
			.setName(`Connected to Raindrop.io as ${this.plugin.settings.username}`)
			.addButton((button) => {
				return button
					.setButtonText('Disconnect')
					.setCta()
					.onClick(async () => {
						button
							.removeCta()
							.setButtonText('Removing API token...')
							.setDisabled(true);

						try {
							this.api.tokenManager.clear();
							this.plugin.settings.isConnected = false;
							this.plugin.settings.username = undefined;
							await this.plugin.saveSettings();
						} catch (e) {
							console.error(e);
							new Notice(`Token removed failed: ${e}`);
							return;
						}

						new Notice('Token removed successfully');
						this.display(); // rerender
					});
			});
	}

	private highlightsFolder(): void {
		new Setting(this.containerEl)
		  .setName('Highlights folder location')
		  .setDesc('Vault folder to use for writing Raindrop.io highlights')
		  .addDropdown((dropdown) => {
			const files = (this.app.vault.adapter as any).files;
			Object.keys(files).forEach((key) => {
				if (files[key].type == 'folder') {
					const folder = files[key].realpath;
					dropdown.addOption(folder, folder);
				}
			})

			return dropdown
				.setValue(this.plugin.settings.highlightsFolder)
				.onChange(async (value) => {
					this.plugin.settings.highlightsFolder = value;
					await this.plugin.saveSettings();
				});
		  });
	  }

	private async collections(): Promise<void> {
		new Setting(this.containerEl)
			.setName('Collections')
			.setDesc('Manage collections to be synced')
			.addButton(button => {
				return button
				.setDisabled(!this.plugin.settings.isConnected)
				.setButtonText('Manage')
				.setCta()
				.onClick(async () => {
					// update for new collections
					const allCollections = await this.api.getCollections();
					this.plugin.updateCollectionSettings(allCollections);

					const collectionsModal = new CollectionsModal(this.app, this.plugin);
					this.display(); // rerender
				});
			});
	}

	private async template(): Promise<void> {
		const templateDescFragment = document
			.createRange()
			.createContextualFragment(templateInstructions);

		new Setting(this.containerEl)
			.setName('Highlights template')
			.setDesc(templateDescFragment)
			.addTextArea((text) => {
				text.inputEl.style.width = '100%';
				text.inputEl.style.height = '450px';
				text.inputEl.style.fontSize = '0.8em';
				text.setValue(this.plugin.settings.template)
					.onChange(async (value) => {
						const isValid = this.renderer.validate(value);

						if (isValid) {
							this.plugin.settings.template = value;
							await this.plugin.saveSettings();
						}

						text.inputEl.style.border = isValid ? '' : '1px solid red';
					});
				return text;
			});
	}

	private resetSyncHistory(): void {
		new Setting(this.containerEl)
		  .setName('Reset sync')
		  .setDesc('Wipe sync history to resync')
		  .addButton((button) => {
			return button
				.setButtonText('Reset')
				.setDisabled(!this.plugin.settings.isConnected)
				.setWarning()
				.onClick(async () => {
					for (let id in this.plugin.settings.syncCollections) {
						const collection = this.plugin.settings.syncCollections[id];
						collection.lastSyncDate = undefined;
					}
					this.plugin.saveSettings();
					new Notice("Sync history reset successfully");
			});
		});
	}

	private dateFormat(): void {
		const descFragment = document
		  .createRange()
		  .createContextualFragment(datetimeInstructions);
	
		new Setting(this.containerEl)
			.setName('Date & time format')
			.setDesc(descFragment)
			.addText((text) => {
				return text
					.setPlaceholder('YYYY-MM-DD HH:mm:ss')
					.setValue(this.plugin.settings.dateTimeFormat)
					.onChange(async (value) => {
					this.plugin.settings.dateTimeFormat = value;
					await this.plugin.saveSettings();
					});
			});
	}

	private autoSyncInterval(): void {
		new Setting(this.containerEl)
			.setName('Auto sync in interval (minutes)')
			.setDesc('Sync every X minutes. To disable auto sync, specify negative value or zero (default)')
			.addText((text) => {
				text
				.setPlaceholder(String(0))
				.setValue(this.plugin.settings.autoSyncInterval.toString())
				.onChange(async (value) => {
					if (!isNaN(Number(value))) {
						const minutes = Number(value);
						this.plugin.settings.autoSyncInterval = minutes;
						await this.plugin.saveSettings();
						console.log("Set raindrop.io autosync interval", minutes);
						if (minutes > 0) {
							this.plugin.clearAutoSync();
							this.plugin.startAutoSync(minutes);
							console.log(
								`Raindrop.io auto sync enabled! Every ${minutes} minutes.`
							);
						} else {
							this.plugin.clearAutoSync();
							console.log("Raindrop.io auto sync disabled!");
						}
					}
				});
			});
	}
}
